#!/bin/bash
#
# Ex.2024-05-10 steven

#今天日期(20xx-xx-xx)
today_date=$(date +%Y%m%d)
#今天是星期幾
today_of_week=$(date +%A)
#今天是幾號
today_day=$(date +%d)


#時間戳 20XX-XX-XX XX:XX:XX
backuptime=$(date +"%F %T")


## 本機 log 存放路徑，超過 14 天 log 會自動被刪除
logpath="/root/iscsi_check"
DATE_14_AGO=`date --date='14 days ago' +"%Y%m%d"`
/bin/rm -f $logpath/$DATE_14_AGO.iscsi_check.log 2> /dev/null

#今天備份log 路徑
bak_log="${logpath}/${today_date}.iscsi_check.log"



mkdir -p $logpath


#function
delete_test_file() {
# 刪除產生測試檔案
if [ -e "/backup/iscsi_check" ]; then
    /bin/rm -f /backup/iscsi_check
fi
if [ -e "/backup/iscsi_check_retry" ]; then
    /bin/rm -f /backup/iscsi_check_retry
fi
echo "$backuptime: 刪除測試檔" >> "$bak_log"  2>/dev/null

}

# 通知信件訊息內容 & 變數
#hdd 的UUID
hdd=`cat /etc/rc.d/rc.local | grep iscsiadm | awk '{print $10}'`
#此主機的hostname
host=`hostname | awk -F. '{print $1}'`
#目前的時間
current_time=$(date +"%T")

#掛載測試
mount_output=$(mount | grep -w "/backup\b")
df_output=$(df -h | grep -w "/backup$")

readonlycheck="0"
readonlycheck_again="0"
read -d '' -r -a evol_array <<EOF
cp24
cp25
cp32
cp35
cp40
cp41
cp42
cp43
cp44
cp45
hd1
hd4
re12
re17
re22
re34
wp4
EOF

evol_array_string=$(printf "%s\n" "${evol_array[@]}")
evol_enable="0"

iscsi_node=`iscsiadm -m session |grep iqn.$host`
if echo ${evol_array_string} | grep -q -w "$host"; then
        if echo "$iscsi_node" | grep -q "$host" && echo "$iscsi_node" | grep -q $host"new"; then
                evol_enable="1"
        else
                :
        fi

else
        :
fi

#===========messages存放區===========

messages_midnight="
$host 目前有 iSCSI Read-only file system 異常，目前已自動重新掛載。

請確認Jetbackup備份排程是否有正常執行。
"
messages_normal="
$host 目前有 iSCSI Read-only file system 異常，目前已自動重新掛載。
"
messages_failed="
$host iSCSI重新掛載異常，請依照以下流程檢查:

1. 於 /backup 試著touch 檔案確認。

2. 如無法建立檔案，則 輸入以下指令:
     
    umount /backup -l
    iscsiadm -m node -T iqn.$host --logout
    iscsiadm -m node -T iqn.$host --login
    mount -U $hdd /backup

   如果目前此主機有舊的iSCSI備份在/backup2 而且新備份在 /backup，請改用以下指令:

    iscsiadm -m node -T iqn.${host}new --logout
    iscsiadm -m node -T iqn.${host}new --login   
    mount -U $hdd /backup

  
    接著使用以下指令確認是否有正常掛載:
   
    mount | grep /backup
    df -h | grep /backup
   
3. 如發生時間是凌晨及早上，請確認Jetbackup備份排程是否有正常執行。

4. 如以上操作後仍無法掛載成功，請通知二線或主管協助確認問題。
"

# 判斷當前時間是否是凌晨~中午12點
if [[ "$current_time" > "00:00:00" && "$current_time" < "12:00:00" ]]; then
    messages="$messages_midnight"
else
    messages="$messages_normal"
fi


#==========================================================================
## 主程式


# 測試產生檔案，確認目前iSCSI掛載是否為 Read-only
if touch /backup/iscsi_check > /dev/null 2>&1; then
     :
else
    error_message=$(touch /backup/iscsi_check 2>&1)	
    if [[ $error_message == *"Read-only file system"* ]]; then
	readonlycheck="1"
    else
	:
    fi
fi

# 如果無法新增檔案，表示目前 /backup 是不可寫入，狀態為 Read-only，則自動重新掛載
if [ "$evol_enable" == "0" ]  && { [ "$readonlycheck" == "1" ] || [ ! -e /backup/iscsi_check ] || [ ! -n "$mount_output" ] || [ ! -n "$df_output" ]; }; then
    umount /backup -l
    echo "   " >> "$bak_log" 2>/dev/null
    echo "$backuptime: iSCSI為 Read-only狀態" >> "$bak_log"  2>/dev/null
    echo "$backuptime: 取消掛載 /backup" >> "$bak_log"  2>/dev/null
    iscsiadm -m node -T iqn.$host --logout
    echo "$backuptime: 登出iscsi session" >> "$bak_log"  2>/dev/null
    iscsiadm -m node -T iqn.$host --login
    echo "$backuptime: 登入iscsi session" >> "$bak_log"  2>/dev/null
    sleep 1
    mount -U $hdd /backup
    echo "$backuptime: 重新掛載 /backup" >> "$bak_log"  2>/dev/null



    # 重新獲取變數值
    mount_output_re=$(mount | grep -w "/backup\b")
    df_output_re=$(df -h | grep -w "/backup$")
    readonlycheck_again="0"

    if touch /backup/iscsi_check_retry > /dev/null 2>&1; then
         :
    else
        error_message=$(touch /backup/iscsi_check_retry 2>&1)
        if [[ $error_message == *"Read-only file system"* ]]; then
            readonlycheck_again="1"
        else
            :
        fi
    fi

    
    #當 touch 檔案再次失敗 以及 判斷 磁區尚未掛載的雙重條件皆不符合時，寄送異常通知信件
    if [ ! -e /backup/iscsi_check_retry ] || [ "$readonlycheck_again" == "1" ] || [ ! -n "$mount_output_re" ] || [ ! -n "$df_output_re" ]; then
         messages="$messages_failed"
	 echo "$messages" | mutt -s `hostname`' iSCSI 自動重新掛載異常通知' support@twnoc.net,jacky@host.com.tw,steven@host.com.tw,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
         echo "$backuptime: iSCSI 自動重新掛載異常通知信件寄送" >> "$bak_log"  2>/dev/null

	 delete_test_file
	 exit
	
    else
	#當一切正常時
	echo "$messages" | mutt -s `hostname`' iSCSI 自動重新掛載通知' jacky@host.com.tw,steven@host.com.tw,support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
         echo "$backuptime: iSCSI 自動重新掛載通知信件寄送" >> "$bak_log"  2>/dev/null

    fi #結束再次掛載判斷

elif [ "$evol_enable" == "1" ]  && { [ "$readonlycheck" == "1" ] || [ ! -e /backup/iscsi_check ] || [ ! -n "$mount_output" ] || [ ! -n "$df_output" ]; }; then
    umount /backup -l
    echo "   " >> "$bak_log" 2>/dev/null
    echo "$backuptime: iSCSI為 Read-only狀態" >> "$bak_log"  2>/dev/null
    echo "$backuptime: 取消掛載 /backup" >> "$bak_log"  2>/dev/null
    iscsiadm -m node -T iqn.${host}new --logout
    echo "$backuptime: 登出iscsi session" >> "$bak_log"  2>/dev/null
    iscsiadm -m node -T iqn.${host}new --login
    echo "$backuptime: 登入iscsi session" >> "$bak_log"  2>/dev/null
    sleep 1
    mount -U $hdd /backup
    echo "$backuptime: 重新掛載 /backup" >> "$bak_log"  2>/dev/null



    # 重新獲取變數值
    mount_output_re=$(mount | grep -w "/backup\b")
    df_output_re=$(df -h | grep -w "/backup$")
    readonlycheck_again="0"
    if touch /backup/iscsi_check_retry > /dev/null 2>&1; then
         :
    else
        error_message=$(touch /backup/iscsi_check_retry 2>&1)
        if [[ $error_message == *"Read-only file system"* ]]; then
            readonlycheck_again="1"
        else
            :
        fi
    fi


    
    #當 touch 檔案再次失敗 以及 判斷 磁區尚未掛載的雙重條件皆不符合時，寄送異常通知信件
    if [ ! -e /backup/iscsi_check_retry ] || [ "$readonlycheck_again" == "1" ] || [ ! -n "$mount_output_re" ] || [ ! -n "$df_output_re" ]; then
         messages="$messages_failed"
	 echo "$messages" | mutt -s `hostname`' iSCSI 自動重新掛載異常通知' support@twnoc.net,jacky@host.com.tw,steven@host.com.tw,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
         echo "$backuptime: iSCSI 自動重新掛載異常通知信件寄送" >> "$bak_log"  2>/dev/null

	 delete_test_file
	 exit
	
    else
	#當一切正常時
	echo "$messages" | mutt -s `hostname`' iSCSI 自動重新掛載通知' jacky@host.com.tw,steven@host.com.tw,support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
         echo "$backuptime: iSCSI 自動重新掛載通知信件寄送" >> "$bak_log"  2>/dev/null

    fi #結束再次掛載判斷


else 
    echo "$backuptime: iSCSI 狀態正常" >> "$bak_log"  2>/dev/null	
fi #結束第一次判斷
delete_test_file

